/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "gwf_error.h"
#include <gtk/gtk.h>
#include <string.h>

static gchar*		get_error		(GwfError *error,gchar *name);
static gboolean		remove_foreach_error	(gpointer key,gpointer value,gpointer user_data);

GwfError*
gwf_error_new (void)
{
  GwfError	*error;

  error = g_malloc(sizeof(GwfError));
  error->errorlist = g_hash_table_new(g_str_hash,g_str_equal);
  error->error = NULL;
  error->func_data = NULL;

  return error;
}

void
gwf_error_free (GwfError *error)
{
  g_return_if_fail(error != NULL);

  g_hash_table_foreach_remove(error->errorlist,remove_foreach_error,NULL);
  g_hash_table_destroy(error->errorlist);
  g_free(error);
}

void
gwf_error_set_func (GwfError *error,GwfErrorFunc func,gpointer func_data)
{
  g_return_if_fail(error != NULL);

  error->error = func;
  error->func_data = func_data;
}

gint
gwf_error_add_error (GwfError *error,gchar *name,GwfErrType type,gchar *message)
{
  gchar		*tmp,*typestr;

  g_return_val_if_fail(error != NULL,0);
  g_return_val_if_fail(name != NULL,0);
  g_return_val_if_fail(message != NULL,0);

  if (get_error(error,name))
    return 0;

  switch (type)
  {
    case GWF_HINT    :	typestr = "HINT:";
    			break;
    case GWF_WARNING :	typestr = "WARNING:";
    			break;
    case GWF_ERROR   :	typestr = "ERROR:";
    			break;
    default	     :  return 0;
    			break;
  }

  tmp = g_strdup_printf("%s%s",typestr,message);
  g_hash_table_insert(error->errorlist,(gpointer)g_strdup(name),(gpointer)tmp);

  return 1;
}

void
gwf_error_remove_error (GwfError *error,gchar *name)
{
  gchar		*origname;
  gchar		*origmsg;

  g_return_if_fail(error != NULL);
  g_return_if_fail(name != NULL);

  if (!g_hash_table_lookup_extended(error->errorlist,(gconstpointer)name,(gpointer*)&origname,(gpointer*)&origmsg))
    return;

  g_hash_table_remove(error->errorlist,(gconstpointer)name);
  g_free(origname);
  g_free(origmsg);
}

void
gwf_error_raise_error (GwfError *error,gchar *name,...)
{
  gchar		*format;
  gchar		*tmp;
  gchar 	*buffer;
  gchar		*typestr;
  GwfErrType	type;
  va_list 	args;

  g_return_if_fail(error != NULL);
  g_return_if_fail(name != NULL);

  format = get_error(error,name);
  if (!format)
  {
    error->error(error->func_data,GWF_ERROR,"Gwf Program Error: The error message could not be found. Contact the developer");
    return;
  }

  va_start (args, name);
  buffer = g_strdup_vprintf (format, args);
  va_end (args);

  typestr = g_strndup(buffer,(gint)(strchr(buffer,':') - buffer));
  if (strcmp(typestr,"HINT") == 0)
    type = GWF_HINT;
  else
    if (strcmp(typestr,"WARNING") == 0)
      type = GWF_WARNING;
    else
      type = GWF_ERROR;
  g_free(typestr);

  tmp = g_strdup_printf("Gwf : %s",strchr(buffer,':') + 1);
  g_free(buffer);

  error->error(error->func_data,type,tmp);
  g_free(tmp);
}


static gchar*
get_error(GwfError *error,gchar *name)
{
  return (gchar*)g_hash_table_lookup(error->errorlist,name);
}

static gboolean
remove_foreach_error(gpointer key,gpointer value,gpointer user_data)
{
  g_free(key);
  g_free(value);
  return TRUE;
}