/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef GWF_SOURCE_H
#define GWF_SOURCE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "gwf_project.h"
#include "gwf_license.h"
#include "gwf_error.h"
#include <gtk/gtk.h>

typedef enum 	_GwfWritePrior		GwfWritePrior;
typedef enum 	_GwfWriteFile		GwfWriteFile;
typedef struct 	_GwfWriteData		GwfWriteData;
typedef struct 	_GwfCheckData		GwfCheckData;
typedef struct	_GwfSource		GwfSource;

typedef void		(*GwfSrcWrtFunc)	(GwfProject *project,GString *src);
typedef gint		(*GwfSrcCheckFunc)	(GwfProject *project,GwfError *error);

enum _GwfWritePrior
{
  COND_DEF_START,
  INCLUDE,
  DEFINE,
  TYPEDEF,
  TYPE,
  EXTERN,
  VARIABEL,
  FUNC_DECL,
  FUNCTION,
  COND_DEF_END
};

enum _GwfWriteFile
{
  HFILE,
  CFILE
};

struct _GwfCheckData
{
  GwfSrcCheckFunc	func;
  gchar			*name;
  gboolean		enabled;
};

struct _GwfWriteData
{
  GwfWritePrior		prior;
  GwfSrcWrtFunc		func;
  gchar			*name;
  gboolean		enabled;
};

struct _GwfSource
{
  GList		*checks;
  GList		*c_funcs;
  GList		*h_funcs;
  GString	*src;
  GwfProject	*project;
  GwfError	*error;
  gint		result;
};

GwfSource*	gwf_source_write_new		(void);
void		gwf_source_write_free		(GwfSource *srcwrt);

void		gwf_source_schedule_func	(GwfSource *srcwrt,gchar *name,GwfWritePrior prior,GwfSrcWrtFunc func,GwfWriteFile file);
void		gwf_source_unschedule_by_name	(GwfSource *srcwrt,gchar *name);
gint		gwf_source_add_check		(GwfSource *srcwrt,gchar* name,GwfSrcCheckFunc func);
void		gwf_source_del_check		(GwfSource *srcwrt,gchar* name);

void		gwf_source_set_by_name		(GwfSource *srcwrt,gchar *name,gboolean enabled);

gint		gwf_source_write_check		(GwfSource *srcwrt,GwfProject *project,GwfError *error);
gchar*		gwf_source_write_h		(GwfSource *srcwrt,GwfProject *project,GwfLicList *liclist);
gchar*		gwf_source_write_c		(GwfSource *srcwrt,GwfProject *project,GwfLicList *liclist);

#ifdef __cplusplus
}
#endif

#endif
