/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "gwf_source_basic.h"
#include "gwf_project.h"
#include "gwf_source.h"
#include "gwf_tools.h"
#include "gwf_source_tools.h"
#include "gwf_error.h"
#include <gtk/gtk.h>

static void		write_get_type_func 		(GwfProject *project,GString *src);
static void		write_class_init_func		(GwfProject *project,GString *src);
static void		write_init_destroy_func 	(GwfProject *project,GString *src);
static void		write_new_construct_func	(GwfProject *project,GString *src);

gint
gwf_source_basic_check (GwfProject *project,GwfError *error)
{
  gint		result = 1;

  g_return_val_if_fail(project != NULL,0);
  g_return_val_if_fail(error != NULL,0);


  if (!gwf_project_get_widname(project))
  {
    gwf_error_raise_error(error,"Source : Basic : No widname");
    result = 0;
  }

  if (!gwf_project_get_prntname(project))
  {
    gwf_error_raise_error(error,"Source : Basic : No parent");
    result =  0;
  }

  return result;
}

void
gwf_source_basic_write_includes_h (GwfProject *project,GString *src)
{
  gchar		*incl;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  incl = gwf_project_get_prntincl(project);
  if (incl)
    g_string_sprintfa(src,"#include %s\n",incl);
}


void
gwf_source_basic_write_cast_defs (GwfProject *project,GString *src)
{
  gchar		*def1,*def2,*def3;
  gchar		*wn1,*wn2,*wname;
  gchar		*funcname,*structname;
  gint		maxlen;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  gwf_source_get_widname_as_def(project,&wn1,&wn2);
  funcname = gwf_source_get_widname_as_func(project);
  structname = gwf_source_get_widname_as_struct(project);

  wname = (wn2) ? g_strdup_printf("%s_%s",wn1,wn2) : g_strdup_printf("%s",wn1);

  def1 = g_strdup_printf("#define %s(obj)",wname);
  def2 = g_strdup_printf("#define %s_CLASS(klass)",wname);
  def3 = (wn2) ? g_strdup_printf("#define %s_IS_%s(obj)",wn1,wn2) : g_strdup_printf("#IS_%s(obj)",wn1);

  g_free(wn1);
  g_free(wn2);
  g_free(wname);

  maxlen = MAX(strlen(def1),strlen(def2));
  maxlen = MAX(maxlen,strlen(def3));

  gwf_source_add_tabs_after(src,def1,maxlen);
  g_string_sprintfa(src,"GTK_CHECK_CAST(obj, %s_get_type (), %s)\n",funcname,structname);
  g_free(def1);

  gwf_source_add_tabs_after(src,def2,maxlen);
  g_string_sprintfa(src,"GTK_CHECK_CLASS_CAST(klass, %s_get_type (), %sClass)\n",funcname,structname);
  g_free(def2);

  gwf_source_add_tabs_after(src,def3,maxlen);
  g_string_sprintfa(src,"GTK_CHECK_TYPE(obj, %s_get_type ())\n\n",funcname);
  g_free(def3);

  g_free(funcname);
  g_free(structname);
}

void
gwf_source_basic_write_widget_tdef (GwfProject *project,GString *src)
{
  gchar		*widname;
  gchar		*tdef_struct,*tdef_class;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  widname = gwf_source_get_widname_as_struct(project);

  tdef_struct = g_strdup_printf("typedef struct _%s",widname);
  tdef_class  = g_strdup_printf("typedef struct _%sClass",widname);

  gwf_source_add_tabs_after(src,tdef_struct,strlen(tdef_class));
  g_string_sprintfa(src,"%s;\n",widname);
  g_free(tdef_struct);

  gwf_source_add_tabs_after(src,tdef_class,strlen(tdef_class));
  g_string_sprintfa(src,"%sClass;\n\n",widname);
}

void
gwf_source_basic_write_widget (GwfProject *project,GString *src)
{
  gchar		*widname;
  gchar		*parent;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  widname = gwf_source_get_widname_as_struct(project);
  parent = gwf_project_get_prntname(project);

  g_string_sprintfa(src,"struct _%s\n",widname);
  g_string_append(src,"{\n");
  g_string_sprintfa(src,"  %s\t\tparent;\n",parent);
  g_string_append(src,"};\n\n");
}

void
gwf_source_basic_write_widget_class (GwfProject *project,GString *src)
{
  gchar		*widname;
  gchar		*parent;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  widname = gwf_source_get_widname_as_struct(project);
  parent = gwf_project_get_prntname(project);

  g_string_sprintfa(src,"struct _%sClass\n",widname);
  g_string_append(src,"{\n");
  g_string_sprintfa(src,"  %sClass\t\tparent_class;\n",parent);
  g_string_append(src,"};\n\n");
}

void
gwf_source_basic_write_func_decl_h (GwfProject *project,GString *src)
{
  gchar		*widname;
  gchar		*structname;
  gchar		*varname;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  widname = gwf_source_get_widname_as_func(project);
  structname = gwf_source_get_widname_as_struct(project);
  varname = gwf_source_get_widname_as_var(project);


  gwf_source_add_tabs_after(src,"guint",strlen(structname) + 1);
  gwf_source_add_tabs_after_until_printf(src,48,"%s_get_type",widname);
  g_string_append(src,"(void);\n");

  gwf_source_add_tabs_after_printf(src,strlen(structname) + 1,"%s*",structname);
  gwf_source_add_tabs_after_until_printf(src,48,"%s_new",widname);
  g_string_append(src,"(void);\n");

  gwf_source_add_tabs_after(src,"void",strlen(structname) + 1);

  gwf_source_add_tabs_after_until_printf(src,48,"%s_construct",widname);
  g_string_sprintfa(src,"(%s *%s);\n\n",structname,varname);

  g_free(widname);
  g_free(structname);
  g_free(varname);
}

void
gwf_source_basic_write_includes_c (GwfProject *project,GString *src)
{
  gchar		*incl;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  incl = gwf_project_get_prntincl(project);
  if (incl)
    g_string_sprintfa(src,"#include %s\n",incl);
}

void
gwf_source_basic_write_func_decl_c (GwfProject *project,GString *src)
{
  gchar		*widname;
  gchar		*structname;
  gchar		*varname;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  widname = gwf_source_get_widname_as_func(project);
  structname = gwf_source_get_widname_as_struct(project);
  varname = gwf_source_get_widname_as_var(project);

  g_string_append(src,"static void\t");
  gwf_source_add_tabs_after_printf(src,strlen(widname) + 11,"%s_class_init",widname);
  g_string_sprintfa(src,"(%sClass *klass);\n",structname);

  g_string_append(src,"static void\t");
  gwf_source_add_tabs_after_printf(src,strlen(widname) + 11,"%s_init",widname);
  g_string_sprintfa(src,"(%s *%s);\n",structname,varname);

  g_string_append(src,"static void\t");
  gwf_source_add_tabs_after_printf(src,strlen(widname) + 11,"%s_destroy",widname);
  g_string_append(src,"(GtkObject *object);\n\n");

  g_free(widname);
  g_free(structname);
  g_free(varname);
}

void
gwf_source_basic_write_vars (GwfProject *project,GString *src)
{
  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  g_string_append(src,"\nstatic GtkObjectClass *parent_class = NULL;\n\n");
}

void
gwf_source_basic_write_funcs (GwfProject *project,GString *src)
{
  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  write_get_type_func(project,src);
  write_class_init_func(project,src);
  write_init_destroy_func(project,src);
  write_new_construct_func(project,src);
}

static void
write_get_type_func (GwfProject *project,GString *src)
{
  gchar		*widname;
  gchar		*structname;
  gchar		*parent;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  widname = gwf_source_get_widname_as_func(project);
  structname = gwf_source_get_widname_as_struct(project);
  parent = gwf_source_get_parent_as_func(project);

  g_string_sprintfa(src,"guint\n%s_get_type (void)\n",widname);
  g_string_sprintfa(src,"{\n  static guint %s_type = 0;\n\n",widname);
  g_string_sprintfa(src,"  if (!%s_type)\n    {\n",widname);
  g_string_sprintfa(src,"      GtkTypeInfo %s_info =\n      {\n        \"%s\",\n",widname,structname);
  g_string_sprintfa(src,"        sizeof (%s),\n        sizeof (%sClass),\n",structname,structname);
  g_string_sprintfa(src,"        (GtkClassInitFunc) %s_class_init,\n        (GtkObjectInitFunc) %s_init,\n",widname,widname);
  g_string_append  (src,"        /* reserved_1 */ NULL,\n        /* reserved_2 */ NULL,\n");
  g_string_append  (src,"        (GtkClassInitFunc) NULL,\n      };\n\n");
  g_string_sprintfa(src,"      %s_type = gtk_type_unique (%s_get_type (), &%s_info);\n",widname,parent,widname);
  g_string_sprintfa(src,"    }\n\n  return %s_type;\n}\n\n",widname);

  g_free(widname);
  g_free(structname);
  g_free(parent);
}

static void
write_class_init_func (GwfProject *project,GString *src)
{
  gchar		*widname;
  gchar		*structname;
  gchar		*parent;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  widname = gwf_source_get_widname_as_func(project);
  structname = gwf_source_get_widname_as_struct(project);
  parent = gwf_source_get_parent_as_func(project);

  g_string_sprintfa(src,"static void\n%s_class_init (%sClass *klass)\n{\n",widname,structname);
  g_string_sprintfa(src,"  GtkObjectClass *object_class;\n\n  object_class = (GtkObjectClass*) klass;\n");
  g_string_sprintfa(src,"  object_class->destroy = %s_destroy;\n  parent_class = gtk_type_class (%s_get_type());\n}\n\n",widname,parent);

  g_free(widname);
  g_free(structname);
  g_free(parent);
}

static void
write_init_destroy_func (GwfProject *project,GString *src)
{
  gchar		*widname;
  gchar		*structname;
  gchar		*varname;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  widname = gwf_source_get_widname_as_func(project);
  structname = gwf_source_get_widname_as_struct(project);
  varname = gwf_source_get_widname_as_var(project);

  /* Init func */
  g_string_sprintfa(src,"static void\n%s_init (%s *%s)\n{\n}\n\n",widname,structname,varname);

  /* Destroy func */
  g_string_sprintfa(src,"static void\n%s_destroy (GtkObject *object)\n{\n",widname);
  g_string_append(src,"\n  parent_class->destroy(object);\n}\n\n");

  g_free(widname);
  g_free(structname);
  g_free(varname);
}

static void
write_new_construct_func (GwfProject *project,GString *src)
{
  gchar		*funcname;
  gchar		*structname;
  gchar		*def1,*def2;
  gchar		*varname;

  g_return_if_fail(project != NULL);
  g_return_if_fail(src != NULL);

  funcname = gwf_source_get_widname_as_func(project);
  structname = gwf_source_get_widname_as_struct(project);
  varname = gwf_source_get_widname_as_var(project);
  gwf_source_get_widname_as_def(project,&def1,&def2);

  /* New func */
  g_string_sprintfa(src,"%s*\n%s_new(void)\n{\n  %s *%s;\n\n",structname,funcname,structname,varname);
  g_string_sprintfa  (src,"  %s = %s",varname,def1);
  if (def2)
    g_string_sprintfa(src,"_%s",def2);
  g_string_sprintfa(src,"( gtk_type_new ( %s_get_type()));\n",funcname);
  g_string_sprintfa(src,"  %s_construct(%s);\n  return %s;\n}\n\n",funcname,varname,varname);

  /* Construct func */
  g_string_sprintfa(src,"void\n%s_construct (%s *%s)\n{\n}\n\n",funcname,structname,varname);

  g_free(funcname);
  g_free(structname);
  g_free(varname);
  g_free(def1);
  g_free(def2);
}
