/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include "gwf_widprop_signal.h"
#include "gwf_projdata_signal.h"
#include <gwf_lib.h>

static void		gwf_widpropsig_class_init	(GwfWidPropSigClass	*klass);
static void		gwf_widpropsig_init		(GwfWidPropSig		*wpsig);
static void		gwf_widpropsig_destroy		(GtkObject		*object);

static void		gwf_widpropsig_disconnect 	(GwfProject *project,GwfWidPropSig *wpsig);
static void		gwf_widpropsig_connect 		(GwfProject *project,GwfWidPropSig *wpsig);
static void		gwf_widpropsig_update		(GwfWidPropSig *wpsig);

/************************************/
/* Project & Project list callbacks */
/************************************/
static void curproj_changed_cb		(GwfProjlist *projlist, gchar *name,gpointer user_data);
static void project_added_cb		(GwfProjlist *projlist, gchar *name,gpointer user_data);
static void offer_sig_added_cb		(GwfProject *project, gchar *name,gpointer user_data);
static void offer_sig_changed_cb	(GwfProject *project, gchar *name,gpointer user_data);
static void offer_sig_removed_cb	(GwfProject *project, gchar *name,gpointer user_data);
static void ofrsig_param_added_cb	(GwfProject *project, gchar *signame, gchar *name,gpointer user_data);
static void ofrsig_param_changed_cb	(GwfProject *project, gchar *signame, gchar *name,gpointer user_data);
static void ofrsig_param_removed_cb	(GwfProject *project, gchar *signame, gchar *name,gpointer user_data);

/*****************/
/* GUI Callbacks */
/*****************/
static void	on_clSignals_select_row		(GtkCList *clist,gint row,gint column,GdkEvent *event,gpointer user_data);
static void	on_clParams_select_row		(GtkCList *clist,gint row,gint column,GdkEvent *event,gpointer user_data);
static void	on_btnSignalAdd_clicked		(GtkButton *button,gpointer user_data);
static void	on_btnSignalDelete_clicked	(GtkButton *button,gpointer user_data);
static void	on_btnSignalClear_clicked	(GtkButton *button,gpointer user_data);
static void	on_btnSignalUpdate_clicked	(GtkButton *button,gpointer user_data);
static void	on_btnParamAdd_clicked		(GtkButton *button,gpointer user_data);
static void	on_btnParamDelete_clicked	(GtkButton *button,gpointer user_data);
static void	on_btnParamClear_clicked	(GtkButton *button,gpointer user_data);
static void	on_btnParamUpdate_clicked	(GtkButton *button,gpointer user_data);

/************************/
/* CList fill Functions */
/************************/
static void	signal_add_clist_foreach	(offersig *sig,gpointer user_data);
static void	param_add_clist_foreach		(sigparam *param,gpointer user_data);

/****************/
/* GUI Creation */
/****************/
static void		gwf_widpropsig_interface 	(GwfWidPropSig *wpsig);
static GtkWidget*	gwf_widpropsig_siglist 		(GwfWidPropSig *wpsig);
static GtkWidget*	gwf_widpropsig_sigelem 		(GwfWidPropSig *wpsig);
static GtkWidget*	gwf_widpropsig_paramlist 	(GwfWidPropSig *wpsig);
static GtkWidget*	gwf_widpropsig_paramelem 	(GwfWidPropSig *wpsig);

guint
gwf_widpropsig_get_type (void)
{
  static guint gwf_widpropsig_type = 0;

  if (!gwf_widpropsig_type)
    {
      GtkTypeInfo gwf_widpropsig_info =
      {
	"GwfWidPropSig",
	sizeof (GwfWidPropSig),
	sizeof (GwfWidPropSigClass),
	(GtkClassInitFunc) gwf_widpropsig_class_init,
	(GtkObjectInitFunc) gwf_widpropsig_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gwf_widpropsig_type = gtk_type_unique (gtk_vbox_get_type(), &gwf_widpropsig_info);
    }

  return gwf_widpropsig_type;
}

static void		
gwf_widpropsig_class_init (GwfWidPropSigClass *klass)
{
  ((GtkObjectClass *)klass)->destroy = gwf_widpropsig_destroy;
}

static void
gwf_widpropsig_init (GwfWidPropSig *wpsig)
{
  gwf_widpropsig_interface (wpsig);
  wpsig->cursigrow = -1;
  wpsig->curparamrow = -1;
}

void
gwf_widpropsig_construct(GwfWidPropSig *wpsig,GwfProjlist *projlist)
{
  wpsig->projlist = projlist;
  gwf_projlist_project_foreach(projlist,(GwfProjlistForeachFunc)gwf_widpropsig_connect,(gpointer)wpsig);

  gtk_signal_connect (GTK_OBJECT (projlist), "curproj_changed",
                      GTK_SIGNAL_FUNC(curproj_changed_cb),
                      (gpointer)wpsig);  
  gtk_signal_connect (GTK_OBJECT (projlist), "project_added",
                      GTK_SIGNAL_FUNC (project_added_cb),
                      (gpointer)wpsig);

  gwf_widpropsig_update(wpsig);
}

GtkWidget*
gwf_widpropsig_new(GwfProjlist *projlist)
{
  GwfWidPropSig		*wpsig;
  
  wpsig = GWF_WIDPROP_SIG(gtk_type_new (gwf_widpropsig_get_type()));
  gwf_widpropsig_construct(wpsig,projlist);
  return GTK_WIDGET ( wpsig );
}

static void
gwf_widpropsig_destroy (GtkObject *object)
{
  gwf_projlist_project_foreach(GWF_WIDPROP_SIG(object)->projlist,(GwfProjlistForeachFunc)gwf_widpropsig_disconnect,
                               (gpointer)object);
  gtk_signal_disconnect_by_data(GTK_OBJECT(GWF_WIDPROP_SIG(object)->projlist),(gpointer)object);
}

static void
gwf_widpropsig_disconnect (GwfProject *project,GwfWidPropSig *wpsig)
{
  gtk_signal_disconnect_by_data( GTK_OBJECT(project), (gpointer)wpsig);
}

static void
gwf_widpropsig_connect (GwfProject *project,GwfWidPropSig *wpsig)
{
  gtk_signal_connect (GTK_OBJECT (project), "offer_signal_added",
     		      GTK_SIGNAL_FUNC (offer_sig_added_cb),
                      (gpointer)wpsig);
  gtk_signal_connect (GTK_OBJECT (project), "offer_signal_changed",
     		      GTK_SIGNAL_FUNC (offer_sig_changed_cb),
                      (gpointer)wpsig);
  gtk_signal_connect (GTK_OBJECT (project), "offer_signal_removed",
     		      GTK_SIGNAL_FUNC (offer_sig_removed_cb ),
                      (gpointer)wpsig);
  gtk_signal_connect (GTK_OBJECT (project), "ofrsig_param_added",
     		      GTK_SIGNAL_FUNC (ofrsig_param_added_cb),
                      (gpointer)wpsig);
  gtk_signal_connect (GTK_OBJECT (project), "ofrsig_param_changed",
     		      GTK_SIGNAL_FUNC (ofrsig_param_changed_cb),
                      (gpointer)wpsig);
  gtk_signal_connect (GTK_OBJECT (project), "ofrsig_param_removed",
     		      GTK_SIGNAL_FUNC (ofrsig_param_removed_cb),
                      (gpointer)wpsig);
}

static void
gwf_widpropsig_update		(GwfWidPropSig *wpsig)
{
  GwfProject	*project;
  gtk_clist_clear(wpsig->siglist);
  gtk_clist_clear(wpsig->paramlist);

  gtk_entry_set_text(wpsig->retval,"");
  gtk_entry_set_text(wpsig->rvtype,"GTK_TYPE_INVALID");
  gtk_entry_set_text(wpsig->signame,"");
  gtk_entry_set_text(wpsig->type,"");
  gtk_entry_set_text(wpsig->paramname,"");
  gtk_entry_set_text(wpsig->gtktype,"GTK_TYPE_INVALID");

  project = gwf_projlist_get_current(wpsig->projlist);
  if (project)
    gwf_project_ofrsig_foreach(project,signal_add_clist_foreach,(gpointer)wpsig);
}

/************************************/
/* Project & Project list callbacks */
/************************************/

static void 
curproj_changed_cb (GwfProjlist *projlist, gchar *name,gpointer user_data)
{
  gwf_widpropsig_update(GWF_WIDPROP_SIG(user_data));
}

static void
project_added_cb	(GwfProjlist *projlist, gchar *name,gpointer user_data)
{
  GwfProject	*project;
  
  project = gwf_projlist_get_project(projlist,name);
  gwf_widpropsig_connect (project,GWF_WIDPROP_SIG(user_data));
}

static void	
offer_sig_added_cb	(GwfProject *project, gchar *name,gpointer user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gchar		*temp[3] = {NULL};
  gint		row;
  
  if (!gwf_projlist_project_is_current(wpsig->projlist,project))
    return;
  
  temp[0] = NULL_TO_EMPTY(gwf_project_ofrsig_get_retval(project,name));
  temp[1] = name;
  temp[2] = gtk_type_to_string(gwf_project_ofrsig_get_rvtype(project,name));

  row = gtk_clist_append(wpsig->siglist,temp);
  gtk_clist_select_row(wpsig->siglist,row,0);
}

static void	
offer_sig_changed_cb	(GwfProject *project, gchar *name,gpointer user_data)
{
  GwfWidPropSig	*wpsig = user_data;

  if (!gwf_projlist_project_is_current(wpsig->projlist,project))
    return;

  g_return_if_fail(wpsig->cursigrow != -1);

  gtk_clist_set_text(wpsig->siglist,wpsig->cursigrow,0,NULL_TO_EMPTY(gwf_project_ofrsig_get_retval(project,name)));
  gtk_clist_set_text(wpsig->siglist,wpsig->cursigrow,1,name);
  gtk_clist_set_text(wpsig->siglist,wpsig->cursigrow,2,gtk_type_to_string(gwf_project_ofrsig_get_rvtype(project,name)));
}

static void
offer_sig_removed_cb	(GwfProject *project, gchar *name,gpointer user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gint		row;
  
  if (!gwf_projlist_project_is_current(wpsig->projlist,project))
    return;
 
  row = gtk_clist_get_row_by_text (wpsig->siglist,1,name);
  if (row == -1)
    g_warning("Can't find signal in clist");
  
  gtk_clist_remove(wpsig->siglist,row);
  gtk_clist_clear(wpsig->paramlist);
  gtk_clist_unselect_all(wpsig->siglist);
  wpsig->cursigrow = -1;
}

static void
ofrsig_param_added_cb	(GwfProject *project, gchar *signame, gchar *name,gpointer user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gchar		*temp[3] = {NULL};
  gint		row;
  
  if (!gwf_projlist_project_is_current(wpsig->projlist,project))
    return;

  row = gtk_clist_get_row_by_text(wpsig->siglist,1,signame);
  if (row != wpsig->cursigrow)
    return;
 
  temp[0] = NULL_TO_EMPTY(gwf_project_ofrsig_param_get_type(project,signame,name));
  temp[1] = name;
  temp[2] = gtk_type_to_string(gwf_project_ofrsig_param_get_gtktype(project,signame,name));
  
  row = gtk_clist_append(wpsig->paramlist,temp);
  gtk_clist_select_row(wpsig->paramlist,row,0);
}

static void
ofrsig_param_changed_cb	(GwfProject *project, gchar *signame, gchar *name,gpointer user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gint		row;
  
  if (!gwf_projlist_project_is_current(wpsig->projlist,project))
    return;

  row = gtk_clist_get_row_by_text(wpsig->siglist,1,signame);
  if (row != wpsig->cursigrow)
    return;

  g_return_if_fail(wpsig->curparamrow != -1);

  gtk_clist_set_text(wpsig->paramlist,wpsig->curparamrow,0,
                     NULL_TO_EMPTY(gwf_project_ofrsig_param_get_type(project,signame,name)));
  gtk_clist_set_text(wpsig->paramlist,wpsig->curparamrow,1,name);
  gtk_clist_set_text(wpsig->paramlist,wpsig->curparamrow,2,
                     gtk_type_to_string(gwf_project_ofrsig_param_get_gtktype(project,signame,name)));
}

static void
ofrsig_param_removed_cb	(GwfProject *project, gchar *signame, gchar *name,gpointer user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gint		row;
  
  if (!gwf_projlist_project_is_current(wpsig->projlist,project))
    return;
 
  row = gtk_clist_get_row_by_text(wpsig->siglist,1,signame);
  if (row != wpsig->cursigrow)
    return;

  row = gtk_clist_get_row_by_text (wpsig->paramlist,1,name);
  if (row == -1)
    g_warning("Can't find param in clist");

  gtk_clist_remove(wpsig->paramlist,row);
  gtk_clist_unselect_all(wpsig->paramlist);
  wpsig->curparamrow = -1;
}

/*****************/
/* GUI Callbacks */
/*****************/

static void
on_clSignals_select_row                (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gchar		*temp = NULL;
  GwfProject	*project;
  
  project = gwf_projlist_get_current(wpsig->projlist);
  if (!project)
    return;

  if (gtk_clist_get_text(clist,row,1,&temp))
  {
    gtk_clist_clear(wpsig->paramlist);
    gwf_project_ofrsig_param_foreach(project,temp,param_add_clist_foreach,(gpointer)wpsig);
    gtk_entry_set_text(wpsig->signame,temp);
  }

  if (gtk_clist_get_text(clist,row,0,&temp))
    gtk_entry_set_text(wpsig->retval,temp);

  if (gtk_clist_get_text(clist,row,2,&temp))
    gtk_entry_set_text(wpsig->rvtype,temp);

  wpsig->cursigrow = row;
}


static void
on_clParams_select_row                 (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gchar		*temp = NULL;
  GwfProject	*project;
  
  project = gwf_projlist_get_current(wpsig->projlist);
  if (!project)
    return;

  if (gtk_clist_get_text(clist,row,0,&temp))
    gtk_entry_set_text(wpsig->type,temp);
    
  if (gtk_clist_get_text(clist,row,1,&temp))
    gtk_entry_set_text(wpsig->paramname,temp);

  if (gtk_clist_get_text(clist,row,2,&temp))
    gtk_entry_set_text(wpsig->gtktype,temp);

  wpsig->curparamrow = row;
}

/*************************************/
/* Signal buttons callback functions */
/*************************************/

static void
on_btnSignalAdd_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  GwfProject	*project;
  offersig	sig;
  gchar		*tmp;
  
  project = gwf_projlist_get_current(wpsig->projlist);
  if (!project)
    return;
  
  sig.name = gtk_editable_get_chars(GTK_EDITABLE(wpsig->signame),0,-1);
  if (!EMPTY_TO_NULL(sig.name))
  {
    g_free(sig.name);
    return;
  }

  sig.retval = gtk_editable_get_chars(GTK_EDITABLE(wpsig->retval),0,-1);
  if (!EMPTY_TO_NULL(sig.retval))
  {
    g_free(sig.retval);
    sig.retval = NULL;
  }

  tmp = gtk_editable_get_chars(GTK_EDITABLE(wpsig->rvtype),0,-1);
  if (!EMPTY_TO_NULL(tmp))
  {
    g_free(tmp);
    tmp = NULL;
  }
  #ifdef WITH_TYPE_GUESSING
    sig.rvtype = (sig.retval) ? find_best_gtk_type(tmp,sig.retval) : string_to_gtk_type(tmp);
  #else
    sig.rvtype = string_to_gtk_type(tmp);
  #endif

  g_free(tmp);

  sig.param = NULL;
  
  gwf_project_add_ofrsig (project,&sig);
  
  g_free(sig.retval);
  g_free(sig.name);
}


static void
on_btnSignalDelete_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gchar		*temp;
  GwfProject	*project;
  
  project = gwf_projlist_get_current(wpsig->projlist);
  if (!project)
    return;

  temp = gtk_clist_get_sel_row_col_text(wpsig->siglist,1);
  if (!temp)
    return;

  gwf_project_del_ofrsig(project,temp);
  g_free(temp);
}

static void
on_btnSignalClear_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropSig	*wpsig = user_data;

  gtk_entry_set_text(wpsig->signame,"");
  gtk_entry_set_text(wpsig->retval,"");
  gtk_entry_set_text(wpsig->rvtype,"GTK_TYPE_INVALID");
}

static void
on_btnSignalUpdate_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gchar		*retval = NULL;
  gchar		*rvtype;
  gchar		*newname = NULL;
  gchar		*name = NULL;
  GwfProject	*project;
  
  project = gwf_projlist_get_current(wpsig->projlist);
  if (!project)
    return;
  
  name = gtk_clist_get_sel_row_col_text(wpsig->siglist,1);
  if (!name)
    return;

  newname = gtk_editable_get_chars(GTK_EDITABLE(wpsig->signame),0,-1);
  if (!EMPTY_TO_NULL(newname))
  {
    g_free(newname);
    return;
  }

  retval = gtk_editable_get_chars(GTK_EDITABLE(wpsig->retval),0,-1);
  if (!EMPTY_TO_NULL(retval))
  {
    g_free(retval);
    retval = NULL;
  }
  
  rvtype = gtk_editable_get_chars(GTK_EDITABLE(wpsig->rvtype),0,-1);
  if (!EMPTY_TO_NULL(rvtype))
  {
    g_free(rvtype);
    rvtype = NULL;
  }


  gwf_project_ofrsig_set_retval(project,name,retval);

  #ifdef WITH_TYPE_GUESSING
    gwf_project_ofrsig_set_rvtype(project,name,(retval) ? find_best_gtk_type(rvtype,retval) : string_to_gtk_type(rvtype));
  #else
    gwf_project_ofrsig_set_rvtype(project,name,string_to_gtk_type(rvtype));
  #endif

  gwf_project_ofrsig_set_name(project,name,newname);
  
  g_free(retval);
  g_free(rvtype);
  g_free(newname);
  g_free(name);
}

/*****************************************/
/* Parameter buttons callbacks functions */
/*****************************************/

static void
on_btnParamAdd_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gchar		*signame;
  GwfProject	*project;
  sigparam	param;
  gchar		*tmp;
  
  project = gwf_projlist_get_current(wpsig->projlist);
  if (!project)
    return;
  
  signame = gtk_clist_get_sel_row_col_text(wpsig->siglist,1);
  if (!signame)
    return;

  param.name = gtk_editable_get_chars(GTK_EDITABLE(wpsig->paramname),0,-1);
  if (!EMPTY_TO_NULL(param.name))
  {
    g_free(param.name);
    return;
  }

  param.type = gtk_editable_get_chars(GTK_EDITABLE(wpsig->type),0,-1);
  if (!EMPTY_TO_NULL(param.type))
  {
    g_free(param.type);
    param.type = NULL;
  }
  
  tmp = gtk_editable_get_chars(GTK_EDITABLE(wpsig->gtktype),0,-1);
  if (!EMPTY_TO_NULL(tmp))
  {
    g_free(tmp);
    tmp = NULL;
  }

  #ifdef WITH_TYPE_GUESSING
    param.gtktype = (param.type) ? find_best_gtk_type(tmp,param.type) : string_to_gtk_type(tmp);
  #else
    param.gtktype = string_to_gtk_type(tmp);
  #endif

  gwf_project_ofrsig_add_param(project,signame,&param);
  
  g_free(param.name);
  g_free(param.type);
  g_free(signame);
  g_free(tmp);
}


static void
on_btnParamDelete_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gchar		*temp;
  gchar		*signame;
  GwfProject	*project;
  
  project = gwf_projlist_get_current(wpsig->projlist);
  if (!project)
    return;
  
  signame = gtk_clist_get_sel_row_col_text(wpsig->siglist,1);
  if (!signame)
    return;
    
  temp = gtk_clist_get_sel_row_col_text(wpsig->paramlist,1);
  if (!temp)
    return;
    
  gwf_project_ofrsig_del_param(project,signame,temp);
  g_free(signame);
  g_free(temp);
}

static void
on_btnParamClear_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropSig	*wpsig = user_data;

  gtk_entry_set_text(GTK_ENTRY(wpsig->paramname),"");
  gtk_entry_set_text(GTK_ENTRY(wpsig->type),"");
  gtk_entry_set_text(GTK_ENTRY(wpsig->gtktype),"GTK_TYPE_INVALID");
}

static void
on_btnParamUpdate_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gchar		*type = NULL;
  gchar		*name = NULL;
  gchar		*gtktype = NULL;
  gchar		*newname = NULL;
  gchar		*signame;
  GwfProject	*project;
  
  project = gwf_projlist_get_current(wpsig->projlist);
  if (!project)
    return;
  
  signame = gtk_clist_get_sel_row_col_text(wpsig->siglist,1);
  if (!signame)
    return;
    
  name = gtk_clist_get_sel_row_col_text(wpsig->paramlist,1);
  if (!name)
    return;
    
  newname = gtk_editable_get_chars(GTK_EDITABLE(wpsig->paramname),0,-1);
  if (!EMPTY_TO_NULL(newname))
  {
    g_free(newname);
    return;
  }

  type = gtk_editable_get_chars(GTK_EDITABLE(wpsig->type),0,-1);
  if (!EMPTY_TO_NULL(type))
  {
    g_free(type);
    type = NULL;
  }
  
  gtktype = gtk_editable_get_chars(GTK_EDITABLE(wpsig->gtktype),0,-1);
  if (!EMPTY_TO_NULL(gtktype))
  {
    g_free(gtktype);
    gtktype = NULL;
  }

  gwf_project_ofrsig_param_set_type(project,signame,name,type);

  #ifdef WITH_TYPE_GUESSING
    gwf_project_ofrsig_param_set_gtktype(project,signame,name,(type) ? find_best_gtk_type(gtktype,type) : string_to_gtk_type(gtktype));
  #else
    gwf_project_ofrsig_param_set_gtktype(project,signame,name,string_to_gtk_type(gtktype));
  #endif

  gwf_project_ofrsig_param_set_name(project,signame,name,newname);

  g_free(type);
  g_free(newname);
  g_free(name);
  g_free(signame);
  g_free(gtktype);
}

/************************/
/* CList fill Functions */
/************************/

static void
signal_add_clist_foreach		(offersig	*signal,
   					 gpointer	user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gchar		*temp[3] = {NULL};
  
  temp[0] = signal->retval;
  temp[1] = signal->name;
  temp[2] = gtk_type_to_string(signal->rvtype);

  gtk_clist_append(wpsig->siglist,temp);  
}

static void
param_add_clist_foreach			(sigparam	*param,
   					 gpointer	user_data)
{
  GwfWidPropSig	*wpsig = user_data;
  gchar		*temp[3] = {NULL};
  
  temp[0] = param->type;
  temp[1] = param->name;
  temp[2] = gtk_type_to_string(param->gtktype);

  gtk_clist_append(wpsig->paramlist,temp);
}


/****************/
/* GUI Creation */
/****************/

static void
gwf_widpropsig_interface (GwfWidPropSig *wpsig)
{
  GtkWidget	*table;
  GtkWidget	*separator;

  table = gtk_table_new (2, 3, FALSE);
  gtk_widget_show (table);
  gtk_box_pack_start(GTK_BOX(wpsig),table,TRUE,TRUE,0);

  separator = gtk_vseparator_new ();
  gtk_widget_show (separator);
  gtk_table_attach (GTK_TABLE (table), separator, 1, 2, 0, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 5, 5);

  gtk_table_attach (GTK_TABLE (table), gwf_widpropsig_siglist(wpsig), 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  gtk_table_attach (GTK_TABLE (table), gwf_widpropsig_sigelem(wpsig), 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 5, 5);

  gtk_table_attach (GTK_TABLE (table), gwf_widpropsig_paramlist(wpsig), 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  gtk_table_attach (GTK_TABLE (table), gwf_widpropsig_paramelem(wpsig), 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 5, 5);
}

static GtkWidget*
gwf_widpropsig_siglist (GwfWidPropSig *wpsig)
{
  GtkWidget	*frame;
  GtkWidget	*scrollwin;
  GtkWidget	*clist;
  GtkWidget	*label;

  frame = gtk_frame_new ("Signals");
  gtk_widget_show (frame);

  scrollwin = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrollwin);
  gtk_container_add (GTK_CONTAINER (frame), scrollwin);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist = gtk_clist_new (3);
  wpsig->siglist = GTK_CLIST(clist);
  gtk_widget_show (clist);
  gtk_container_add (GTK_CONTAINER (scrollwin), clist);
  gtk_clist_set_column_width (GTK_CLIST (clist), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist), 1, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist), 2, 80);
  gtk_clist_column_titles_show (GTK_CLIST (clist));

  gtk_signal_connect (GTK_OBJECT (clist), "select_row",
                      GTK_SIGNAL_FUNC (on_clSignals_select_row),
                      (gpointer)wpsig);

  label = gtk_label_new ("Return value");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 0, label);

  label = gtk_label_new ("Name");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 1, label);
  
  label = gtk_label_new ("Return type");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 2, label);

  return frame;
}

static GtkWidget*
gwf_widpropsig_sigelem (GwfWidPropSig *wpsig)
{
  GtkWidget	*frame;
  GtkWidget	*table;
  GtkWidget	*label;
  GtkWidget	*entry;
  GtkWidget	*hbox;
  GtkWidget	*button;
  GtkWidget	*combo;
  GList		*tmp = NULL;
  gint		i;
  
  frame = gtk_frame_new ("Signal");
  gtk_widget_show (frame);

  table = gtk_table_new (3, 3, FALSE);
  gtk_widget_show (table);
  gtk_container_add (GTK_CONTAINER (frame), table);

  label = gtk_label_new ("Return value :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);


  entry = gtk_entry_new ();
  wpsig->retval = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);



  label = gtk_label_new ("Name :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  entry = gtk_entry_new ();
  wpsig->signame = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);


  label = gtk_label_new ("Return Type :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  combo = gtk_combo_new();
  wpsig->rvtype = GTK_ENTRY(GTK_COMBO(combo)->entry);
  gtk_combo_set_value_in_list(GTK_COMBO(combo),TRUE,FALSE);
  gtk_widget_show(combo);
  gtk_table_attach (GTK_TABLE (table), combo, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  for (i = 0; i <= GTK_TYPE_FUNDAMENTAL_LAST ; i++)
    tmp = g_list_append(tmp,gtk_type_to_string(i));

  gtk_combo_set_popdown_strings(GTK_COMBO(combo),tmp);

  g_list_free(tmp);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_table_attach (GTK_TABLE (table), hbox, 0, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  button = gtk_button_new_with_label ("Add");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnSignalAdd_clicked),
                      (gpointer)wpsig);

  button = gtk_button_new_with_label ("Update");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnSignalUpdate_clicked),
                      (gpointer)wpsig);

  button = gtk_button_new_with_label ("Delete");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnSignalDelete_clicked),
                      (gpointer)wpsig);

  button = gtk_button_new_with_label ("Clear");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnSignalClear_clicked),
                      (gpointer)wpsig);
                      
  return frame;
}

static GtkWidget*
gwf_widpropsig_paramlist (GwfWidPropSig *wpsig)
{
  GtkWidget	*frame;
  GtkWidget	*scrollwin;
  GtkWidget	*clist;
  GtkWidget	*label;

  frame = gtk_frame_new ("Parameters");
  gtk_widget_show (frame);

  scrollwin = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrollwin);
  gtk_container_add (GTK_CONTAINER (frame), scrollwin);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist = gtk_clist_new (3);
  wpsig->paramlist = GTK_CLIST(clist);
  gtk_widget_show (clist);
  gtk_container_add (GTK_CONTAINER (scrollwin), clist);
  gtk_clist_set_column_width (GTK_CLIST (clist), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist), 1, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist), 2, 80);
  gtk_clist_column_titles_show (GTK_CLIST (clist));

  gtk_signal_connect (GTK_OBJECT (clist), "select_row",
                      GTK_SIGNAL_FUNC (on_clParams_select_row),
                      (gpointer)wpsig);
                      
  label = gtk_label_new ("Type");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 0, label);

  label = gtk_label_new ("Name");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 1, label);

  label = gtk_label_new ("Gtk Type");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 2, label);
  return frame;
}

static GtkWidget*
gwf_widpropsig_paramelem (GwfWidPropSig *wpsig)
{
  GtkWidget	*frame;
  GtkWidget	*table;
  GtkWidget	*label;
  GtkWidget	*entry;
  GtkWidget	*hbox;
  GtkWidget	*button;
  GtkWidget	*combo;
  GList		*tmp = NULL;
  gint		i;

  frame = gtk_frame_new ("Parameter");
  gtk_widget_show (frame);

  table = gtk_table_new (3, 3, FALSE);
  gtk_widget_show (table);
  gtk_container_add (GTK_CONTAINER (frame), table);

  label = gtk_label_new ("Type :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  entry = gtk_entry_new ();
  wpsig->type = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  label = gtk_label_new ("Name :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  entry = gtk_entry_new ();
  wpsig->paramname = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  label = gtk_label_new ("Gtk Type :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  combo = gtk_combo_new();
  wpsig->gtktype = GTK_ENTRY(GTK_COMBO(combo)->entry);
  gtk_combo_set_value_in_list(GTK_COMBO(combo),TRUE,FALSE);
  gtk_widget_show(combo);
  gtk_table_attach (GTK_TABLE (table), combo, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  for (i = 0; i <= GTK_TYPE_FUNDAMENTAL_LAST ; i++)
    tmp = g_list_append(tmp,gtk_type_to_string(i));

  gtk_combo_set_popdown_strings(GTK_COMBO(combo),tmp);

  g_list_free(tmp);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_table_attach (GTK_TABLE (table), hbox, 0, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  button = gtk_button_new_with_label ("Add");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnParamAdd_clicked),
                      (gpointer)wpsig);

  button = gtk_button_new_with_label ("Update");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);


  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnParamUpdate_clicked),
                      (gpointer)wpsig);

  button = gtk_button_new_with_label ("Delete");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnParamDelete_clicked),
                      (gpointer)wpsig);
                      
  button = gtk_button_new_with_label ("Clear");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 5);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnParamClear_clicked),
                      (gpointer)wpsig);
                      
  return frame;
}

